package grails.plugins.dto.annotation;

import static java.lang.annotation.RetentionPolicy.RUNTIME;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.Target;

@Retention(RUNTIME)
@Target(ElementType.FIELD)
public @interface DtoSummary {
	 /**
     * The name to use, if not specified use the same name as Field being annotated
     * @return
     */
    String name() default "";
    
    /**
     * If a property of the association should be used. Use the dot syntax to signal
     * traversal. E.g. given a Domain class:
     * <pre>
     * 
     * </pre>
     * @return
     */
    String subProperty() default "";

    /**
     * The type name to use, if not specified use the same type as Field being annotated.
     * If possible use the Class based type() parameter. Use this parameter only if the class
     * is not reachable.
     * @return
     */
    String typeName() default "";
    
    /**
     * The type to use, if not specified use the same type as Field being annotated
     * @return
     */
    Class<?> type() default DEFAULT.class;

    /**
     * Since null values are not allowed as default return values, use this helper class
     * @author Ruediger Schobbert, emesit GmbH & Co. KG
     */
    public static final class DEFAULT {}

}
